import esbuild from "esbuild";
import * as path from "path";
import { readFileSync } from "fs";

const parse = (it) => {
  if (it.endsWith("__drizzle__")) {
    const clean = it.slice(-"__drizzle__".length);
    return { drizzle: true, clean, original: it };
  }
  return { drizzle: false, clean: it };
};

export function resolve(specifier, context, nextResolve) {
  const { drizzle, clean } = parse(specifier);
  if (drizzle && !specifier.endsWith(".ts") && !specifier.endsWith(".mts")) {
    return nextResolve(clean);
  }

  if (drizzle) {
    return {
      shortCircuit: true,
      url: specifier,
    };
  }

  const parsedParent = parse(context.parentURL);
  const parentURL = parsedParent.drizzle
    ? new URL(`file://${path.resolve(parsedParent.clean)}`)
    : context.parentURL;

  // Let Node.js handle all other specifiers.
  return nextResolve(specifier, { ...context, parentURL });
}

export async function load(url, context, defaultLoad) {
  const { drizzle, clean } = parse(url);
  if (drizzle) {
    const file = readFileSync(clean, "utf-8");
    if (path.endsWith(".ts") || path.endsWith(".mts")) {
      const source = esbuild.transformSync(file, {
        loader: "ts",
        format: "esm",
      });
      return {
        format: "module",
        shortCircuit: true,
        source: source.code,
      };
    }
  }

  // let Node.js handle all other URLs
  return defaultLoad(url, context, defaultLoad);
}
